import os
import pathlib
import shutil
import unittest

import ddeutil.io.__base.files as fl


class OpenDirTestCase(unittest.TestCase):
    root_path: str

    @classmethod
    def setUpClass(cls) -> None:
        _root_path: str = os.path.dirname(os.path.abspath(__file__)).replace(
            os.sep, "/"
        )
        os.makedirs(f"{_root_path}/open_dir", exist_ok=True)
        os.makedirs(f"{_root_path}/open_dir/data", exist_ok=True)

        cls.root_path: str = f"{_root_path}/open_dir"
        cls.data_path: str = f"{_root_path}/open_dir/data"

        with open(
            pathlib.Path(f"{_root_path}/open_dir/data/test_file.json"),
            mode="w",
        ) as f:
            f.write('{"key": "value"}')
        with open(
            pathlib.Path(f"{_root_path}/open_dir/data/test_file_2.json"),
            mode="w",
        ) as f:
            f.write('{"foo": "bar"}')

    def setUp(self) -> None:
        self.encoding = "utf-8"

    def test_open_dir_common_zip(self):
        data_dir = pathlib.Path(self.data_path)

        opd = fl.Dir(
            path=pathlib.Path(f"{self.root_path}/test_common_dir.zip"),
            compress="zip",
        )
        with opd.open(mode="w") as d:
            for data in data_dir.rglob("*"):
                d.write(filename=data, arcname=data.relative_to(data_dir))

    def test_open_dir_common_tar(self):
        data_dir = pathlib.Path(self.data_path)

        opd = fl.Dir(
            path=pathlib.Path(f"{self.root_path}/test_common_dir.tar.gz"),
            compress="tar",
        )
        with opd.open(mode="w") as d:
            for data in data_dir.rglob("*"):
                d.add(name=data, arcname=data.relative_to(data_dir))

    @classmethod
    def tearDownClass(cls) -> None:
        shutil.rmtree(cls.root_path)



import os
import unittest
import warnings
from textwrap import dedent

import ddeutil.io.__base.files as fl
import yaml


class YamlFileTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.root_path: str = os.path.dirname(os.path.abspath(__file__)).replace(
            os.sep, "/"
        )

    def setUp(self) -> None:
        warnings.simplefilter("ignore", category=ResourceWarning)
        self.yaml_str: str = dedent(
            """
        main_key:
            sub_key:
                string: 'test ${DEMO_ENV_VALUE} value'
                int: 0.001
                bool: false
                list: ['i1', 'i2', 'i3']
        """
        ).strip()
        self.yaml_data: dict = {
            "main_key": {
                "sub_key": {
                    "string": "test ${DEMO_ENV_VALUE} value",
                    "int": 0.001,
                    "bool": False,
                    "list": ["i1", "i2", "i3"],
                }
            }
        }

    def test_write_yaml_file_with_safe_mode(self):
        yaml_path: str = f"{self.root_path}/test_write_file.yaml"

        fl.YamlFl(path=yaml_path).write(self.yaml_data)

        self.assertTrue(os.path.exists(yaml_path))

        os.remove(yaml_path)

    def test_read_yaml_file_with_safe_mode(self):
        yaml_path: str = f"{self.root_path}/test_read_file.yaml"

        with open(yaml_path, mode="w", encoding="utf-8") as f:
            yaml.dump(yaml.safe_load(self.yaml_str), f)

        data = fl.YamlFl(path=yaml_path).read()
        self.assertDictEqual(self.yaml_data, data)

        os.remove(yaml_path)


class YamlEnvFileTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.root_path: str = os.path.dirname(os.path.abspath(__file__)).replace(
            os.sep, "/"
        )

    def setUp(self) -> None:
        self.maxDiff = None
        warnings.simplefilter("ignore", category=ResourceWarning)
        self.yaml_str: str = dedent(
            """
        main_key:
            sub_key:
                key01: 'test ${DEMO_ENV_VALUE} value'
                key02: $1 This is escape with number
                key03: $$ESCAPE This is escape with $
                key04: ['i1', 'i2', '${DEMO_ENV_VALUE}']
                key05: ${DEMO_ENV_VALUE_EMPTY:default}
                key06: $${DEMO_ENV_VALUE}
        """
        ).strip()
        self.yaml_data: dict = {
            "main_key": {
                "sub_key": {
                    "key01": "test demo value",
                    "key02": "$1 This is escape with number",
                    "key03": "$ESCAPE This is escape with $",
                    "key04": ["i1", "i2", "demo"],
                    "key05": "default",
                    "key06": "${DEMO_ENV_VALUE}",
                }
            }
        }

    def test_read_yaml_file_with_safe_mode(self):
        yaml_path: str = f"{self.root_path}/test_read_file_env.yaml"

        with open(yaml_path, mode="w", encoding="utf-8") as f:
            yaml.dump(yaml.safe_load(self.yaml_str), f)

        os.environ["DEMO_ENV_VALUE"] = "demo"

        data = fl.YamlEnvFl(path=yaml_path).read()
        self.assertDictEqual(self.yaml_data, data)

        os.remove(yaml_path)

    def test_read_yaml_file_with_safe_mode_and_prepare(self):
        yaml_path: str = f"{self.root_path}/test_read_file_env_prepare.yaml"

        with open(yaml_path, mode="w", encoding="utf-8") as f:
            yaml.dump(yaml.safe_load(self.yaml_str), f)

        os.environ["DEMO_ENV_VALUE"] = "demo"

        yml_loader = fl.YamlEnvFl(path=yaml_path)
        yml_loader.prepare = lambda x: f"{x}!!"
        data = yml_loader.read()
        self.assertDictEqual(
            {
                "main_key": {
                    "sub_key": {
                        "key01": "test demo!! value",
                        "key02": "$1 This is escape with number",
                        "key03": "$ESCAPE This is escape with $",
                        "key04": ["i1", "i2", "demo!!"],
                        "key05": "default!!",
                        "key06": "${DEMO_ENV_VALUE}",
                    }
                }
            },
            data,
        )

        os.remove(yaml_path)

    def test_read_yaml_file_with_safe_mode_and_prepare_2(self):
        yaml_path: str = f"{self.root_path}/test_read_file_env_prepare_2.yaml"

        with open(yaml_path, mode="w", encoding="utf-8") as f:
            yaml.dump(yaml.safe_load(self.yaml_str), f)

        os.environ["DEMO_ENV_VALUE"] = "P@ssW0rd"

        import urllib.parse

        yml_loader = fl.YamlEnvFl
        yml_loader.prepare = staticmethod(
            lambda x: urllib.parse.quote_plus(str(x))
        )
        data = yml_loader(path=yaml_path).read()
        self.assertDictEqual(
            {
                "main_key": {
                    "sub_key": {
                        "key01": "test P%40ssW0rd value",
                        "key02": "$1 This is escape with number",
                        "key03": "$ESCAPE This is escape with $",
                        "key04": ["i1", "i2", "P%40ssW0rd"],
                        "key05": "default",
                        "key06": "${DEMO_ENV_VALUE}",
                    }
                }
            },
            data,
        )

        os.remove(yaml_path)



import shutil
from pathlib import Path

import ddeutil.io.config as conf
import pytest


@pytest.fixture(scope="module")
def target_path(test_path) -> Path:
    return test_path / "conf_file_temp"


def test_base_conf_read_file(demo_path, target_path):
    bcf = conf.BaseConfFl(demo_path)

    assert {
        "alias": "conn_local_file",
        "endpoint": "file:///null/tests/examples/dummy",
        "type": "connection.LocalFileStorage",
    } == bcf.load(name="conn_local_file")

    bcf.move(
        "demo_01_connections.yaml",
        destination=target_path / "demo_01_connections.yaml",
    )

    bcf_temp = conf.BaseConfFl(target_path)
    assert {
        "alias": "conn_local_file",
        "endpoint": "file:///null/tests/examples/dummy",
        "type": "connection.LocalFileStorage",
    } == bcf_temp.load(name="conn_local_file")

    assert (target_path / "demo_01_connections.yaml").exists()

    if target_path.exists():
        shutil.rmtree(target_path)


def test_conf_read_file(demo_path, target_path):
    cf = conf.ConfFl(demo_path)
    cf.move(
        path="demo_01_connections.yaml",
        destination=target_path / "demo_01_connections.yaml",
    )

    _stage_path = target_path / "demo_01_connections_stage.json"

    cf.create(path=_stage_path)
    assert _stage_path.exists()
    cf.save_stage(path=_stage_path, data=cf.load("conn_local_file"))

    assert {
        "alias": "conn_local_file",
        "endpoint": "file:///null/tests/examples/dummy",
        "type": "connection.LocalFileStorage",
    } == cf.load_stage(path=_stage_path)

    cf.save_stage(
        path=_stage_path,
        data={"temp_additional": cf.load("conn_local_file")},
        merge=True,
    )

    cf.remove_stage(
        path=_stage_path,
        name="temp_additional",
    )

    assert {
        "alias": "conn_local_file",
        "endpoint": "file:///null/tests/examples/dummy",
        "type": "connection.LocalFileStorage",
    } == cf.load_stage(path=_stage_path)

    if target_path.exists():
        shutil.rmtree(target_path)



# ------------------------------------------------------------------------------
# Copyright (c) 2022 Korawich Anuttra. All rights reserved.
# Licensed under the MIT License. See LICENSE in the project root for
# license information.
# ------------------------------------------------------------------------------
from __future__ import annotations

from typing import (
    Any,
    Callable,
    Union,
)

from ddeutil.core import import_string, str2args

from .__base import RegexConf
from .exceptions import ConfigArgumentError


def map_secret(
    value: Any,
    secrets: dict[str, Any],
) -> Union[dict, str, Any]:
    """Map the secret value to configuration data.

    :param value:
    :param secrets:
    """
    if isinstance(value, dict):
        return {k: map_secret(value[k], secrets) for k in value}
    elif isinstance(value, (list, tuple)):
        return type(value)([map_secret(i, secrets) for i in value])
    elif not isinstance(value, str):
        return value
    for search in RegexConf.RE_SECRETS.finditer(value):
        searches: dict = search.groupdict()
        if "." in (br := searches["braced"]):
            raise ConfigArgumentError(
                "secrets",
                f", value {br!r},  should not contain dot ('.') in get value.",
            )
        value: str = value.replace(
            searches["search"],
            secrets.get(br.strip(), searches["braced_default"]),
        )
    return value


def map_function(value: Any) -> Union[dict, str, Any]:
    """Map the function result to configuration data."""
    if isinstance(value, dict):
        return {k: map_function(value[k]) for k in value}
    elif isinstance(value, (list, tuple)):
        return type(value)([map_function(i) for i in value])
    elif not isinstance(value, str):
        return value
    for search in RegexConf.RE_FUNCTION.finditer(value):
        searches: dict = search.groupdict()
        if not callable(_fn := import_string(searches["function"])):
            raise ConfigArgumentError(
                "@function",
                f'from function {searches["function"]!r} is not callable.',
            )
        args, kwargs = str2args(searches["arguments"])
        value: str = value.replace(searches["search"], _fn(*args, **kwargs))
    return value


def map_func_to_str(value: Any, fn: Callable[[str], str]) -> Any:
    """Map any function from input argument to configuration data."""
    if isinstance(value, dict):
        return {k: map_func_to_str(value[k], fn) for k in value}
    elif isinstance(value, (list, tuple)):
        return type(value)([map_func_to_str(i, fn) for i in value])
    elif not isinstance(value, str):
        return value
    return fn(value)



import pathlib

import ddeutil.io.models as md


def test_model_path_data():
    p = md.PathData.model_validate(
        {
            "data": pathlib.Path("."),
            "conf": pathlib.Path("."),
            "archive": pathlib.Path("."),
        }
    )

    assert {
        "data": pathlib.Path("."),
        "conf": pathlib.Path("."),
        "archive": pathlib.Path("."),
        "root": pathlib.Path("."),
    } == p.model_dump()


def test_model_path_data_with_root():
    p = md.PathData.model_validate({"root": "./src/"})
    assert {
        "data": pathlib.Path("./src/data"),
        "conf": pathlib.Path("./src/conf"),
        "archive": pathlib.Path("./src/.archive"),
        "root": pathlib.Path("./src/"),
    } == p.model_dump()


def test_model_rule_data():
    assert {
        "timestamp": {},
        "version": None,
        "excluded": [],
        "compress": None,
    } == md.RuleData.model_validate({}).model_dump()


def test_model_stage_data():
    stage = md.StageData.model_validate(
        {
            "alias": "persisted",
            "format": "{timestamp:%Y-%m-%d}{naming:%c}.json",
            "rules": {
                "timestamp": {"minutes": 15},
            },
        }
    )

    assert {
        "alias": "persisted",
        "format": "{timestamp:%Y-%m-%d}{naming:%c}.json",
        "rules": {
            "timestamp": {"minutes": 15},
            "version": None,
            "excluded": [],
            "compress": None,
        },
        "layer": 0,
    } == stage.model_dump()


def test_model_params():
    params = md.Params.model_validate(
        {
            "stages": {
                "raw": {"format": "{naming:%s}.{timestamp:%Y%m%d_%H%M%S}"},
                "persisted": {"format": "{naming:%s}.{version:v%m.%n.%c}"},
                "curated": {"format": "{domain:%s}_{naming:%s}.{compress:%-g}"},
            }
        }
    )
    assert {
        "stages": {
            "raw": {
                "format": "{naming:%s}.{timestamp:%Y%m%d_%H%M%S}",
                "alias": "raw",
                "rules": {
                    "timestamp": {},
                    "version": None,
                    "excluded": [],
                    "compress": None,
                },
                "layer": 1,
            },
            "persisted": {
                "format": "{naming:%s}.{version:v%m.%n.%c}",
                "alias": "persisted",
                "rules": {
                    "timestamp": {},
                    "version": None,
                    "excluded": [],
                    "compress": None,
                },
                "layer": 2,
            },
            "curated": {
                "format": "{domain:%s}_{naming:%s}.{compress:%-g}",
                "alias": "curated",
                "rules": {
                    "timestamp": {},
                    "version": None,
                    "excluded": [],
                    "compress": None,
                },
                "layer": 3,
            },
        },
        "engine": {
            "values": {
                "dt_fmt": "%Y-%m-%d %H:%M:%S",
                "excluded": ("version", "updt"),
            },
            "flags": {
                "archive": False,
                "auto_update": False,
            },
            "paths": {
                "archive": pathlib.Path(".archive"),
                "conf": pathlib.Path("conf"),
                "data": pathlib.Path("data"),
                "root": pathlib.Path("."),
            },
        },
    } == params.model_dump()
